% Launches the DVF plotting window and runs the plotting task.
function dvf_launch(amp_max, rpm_max)

clc;
if (nargin < 1)
    amp_max = 2000; rpm_max = 5000;
end

% FIGURE SETUP
% ---------------------------------------------

% Setup the Bode plot figure, get handles
f           = launch_gui;
x_mag       = findobj(f, 'Tag', 'x_mag');
x_phs       = findobj(f, 'Tag', 'x_phs');
x_check     = findobj(f, 'Tag', 'x_check');
y_mag       = findobj(f, 'Tag', 'y_mag');
y_phs       = findobj(f, 'Tag', 'y_phs');
y_check     = findobj(f, 'Tag', 'y_check');

% Initialize plot variables
rpm       = [];
x_mag_dat = [];
x_phs_dat = [];
y_mag_dat = [];
y_phs_dat = [];

% Set up plot axis and labels
axes(x_mag); 
ylim([0 amp_max]); xlim([0 rpm_max]); grid on; hold on;
ylabel('Amplitude (um)'); title('X Channel','FontSize', 16);

axes(x_phs); 
ylim([-360 0]); xlim([0 rpm_max]); grid on; hold on;
ylabel('Phase (deg)'); xlabel('Frequency (RPM)');

axes(y_mag); 
ylim([0 amp_max]); xlim([0 rpm_max]); grid on; hold on;
ylabel('Amplitude (um)'); title('Y Channel','FontSize', 16);

axes(y_phs); 
ylim([-360 0]); xlim([0 rpm_max]); grid on; hold on;
ylabel('Phase (deg)'); xlabel('Frequency (RPM)');

% DATA COLLECTION LOOP
% Run at about 20 FPS
% ---------------------------------------------

while (1)
    tic;
    pause(1/20);
    if (~ishandle(f)); break; end; drawnow;
    if (isempty(f.UserData.serial)); continue; end;
    
    % Get full packet
    ser = f.UserData.serial;
    if ser.BytesAvailable > 30
        read = fscanf(ser, '%d %d %d %d %d');
        if (x_check.Value)
            plot(read(1),  read(2), 'bd', 'Parent', x_mag, 'MarkerSize', 3);
            plot(read(1), -read(3), 'bd', 'Parent', x_phs, 'MarkerSize', 3);
        end
        if (y_check.Value)
            plot(read(1),  read(4), 'rd', 'Parent', y_mag, 'MarkerSize', 3);
            plot(read(1), -read(5), 'rd', 'Parent', y_phs, 'MarkerSize', 3);
        end
    end
end

% Just some GUI launching minutia - initializing the handles struct, etc.
function f = launch_gui
    f           = open('plot_gui.fig');
    f.UserData  = struct('serial', []);
    handles = guihandles(f); 
    guidata(f, handles);
    
    % Close any existing connection formed by MATLAB
    inst = instrfind;
    isOpen = @(x) strcmp(x, 'open');
    if ~isempty(inst)
        if iscell(inst)
        toClose = inst(cellfun(isOpen, inst.Status));
        else
            toClose = inst(isOpen(inst.Status));
        end
        for i = 1:length(toClose)
           fclose(toClose(i)); 
        end
    end